#include "psp_api.h"
#include "bsp_external_interrupts.h"
#include "psp_ext_interrupts_eh1.h"
#include "bsp_timer.h"
#include "bsp_printf.h"

#define SegEn_ADDR      0x80001038
#define SegDig_ADDR     0x8000103C

#define GPIO_SWs        0x80001400
#define GPIO_LEDs       0x80001404
#define GPIO_INOUT      0x80001408
#define RGPIO_INTE      0x8000140C
#define RGPIO_PTRIG     0x80001410
#define RGPIO_CTRL      0x80001418
#define RGPIO_INTS      0x8000141C

#define RPTC_CNTR       0x80001200
#define RPTC_HRC        0x80001204
#define RPTC_LRC        0x80001208
#define RPTC_CTRL       0x8000120c

#define Select_INT      0x80001018

int SegDisplCount=0;

extern D_PSP_DATA_SECTION D_PSP_ALIGNED(1024) pspInterruptHandler_t G_Ext_Interrupt_Handlers[8];


void GPIO_ISR(void)
{
  unsigned int i;

  /* LEDe yaz */
  i = M_PSP_READ_REGISTER_32(GPIO_LEDs);             /* RGPIO_OUT */
  i = !i;                                            /* LEDleri tersine döndür */
  i = i & 0x1;                                       /* Yalnızca en-sağdaki LEDi tut */
  M_PSP_WRITE_REGISTER_32(GPIO_LEDs, i);             /* RGPIO_OUT */

  /* Genel-Amaçlı Girdi/Çıktı kesintisini boşalt */
  M_PSP_WRITE_REGISTER_32(RGPIO_INTS, 0x0);           /* RGPIO_INTS */

  /* Belirli dış kesintinin oluşmasını durdur */
  bspClearExtInterrupt(4);
}


void DefaultInitialization(void)
{
  u32_t uiSourceId;

  /* Kesinti vektorünü kaydet */
  pspInterruptsSetVectorTableAddress(&M_PSP_VECT_TABLE);

  /* MEIVT CSR'da dış-kesintilerin vektör-çizelge adresini ayarla */
  pspExternalInterruptSetVectorTableAddress(G_Ext_Interrupt_Handlers);

  /* Oluşma-Yazmacını ilk durumuna koy (oluşan dış kesinti yok) */
  bspInitializeGenerationRegister(D_PSP_EXT_INT_ACTIVE_HIGH);

  for (uiSourceId = D_BSP_FIRST_IRQ_NUM; uiSourceId <= D_BSP_LAST_IRQ_NUM; uiSourceId++)
  {
    /* Dış-kesinti tetiklerinin boşaltıldığının sağlamasını yap */
    bspClearExtInterrupt(uiSourceId);
  }

  /* Standart öncelik yerlerini ayarla */
  pspExtInterruptSetPriorityOrder(D_PSP_EXT_INT_STANDARD_PRIORITY);

  /* Kesintilerin eşiklerini alçağa ayarla (== bütün kesintiler sunulmalı) */
  pspExtInterruptsSetThreshold(M_PSP_EXT_INT_THRESHOLD_UNMASK_ALL_VALUE);

  /* Yuvalanma öncelik eşiklerini alçağa ayarla (== bütün kesintiler sunulmalı) */
  pspExtInterruptsSetNestingPriorityThreshold(M_PSP_EXT_INT_THRESHOLD_UNMASK_ALL_VALUE);
}


void ExternalIntLine_Initialization(u32_t uiSourceId, u32_t priority, pspInterruptHandler_t pTestIsr)
{
  /* Geçit Kesinti türünü (Düzey) ayarla */
  pspExtInterruptSetType(uiSourceId, D_PSP_EXT_INT_LEVEL_TRIG_TYPE);

  /* Geçit Ucaylılığını (Etkin yüksek) ayarla */
  pspExtInterruptSetPolarity(uiSourceId, D_PSP_EXT_INT_ACTIVE_HIGH);

  /* Geçidi boşalt */
  pspExtInterruptClearPendingInt(uiSourceId);

  /* IRQ4 önceliğini ayarla */
  pspExtInterruptSetPriority(uiSourceId, priority);
    
  /* IRQ4 interrupts in the PIC'de IRQ4 kesintilerini etkinleştir */
  pspExternalInterruptEnableNumber(uiSourceId);

  /* ISR'ı kaydet */
  G_Ext_Interrupt_Handlers[uiSourceId] = pTestIsr;
}


void GPIO_Initialization(void)
{
  /* LEDler ile Anahtarları yapılandır */
  M_PSP_WRITE_REGISTER_32(GPIO_INOUT, 0xFFFF);        /* GPIO_INOUT */
  M_PSP_WRITE_REGISTER_32(GPIO_LEDs, 0x0);            /* GPIO_LEDs */

  /* Genel-Amaçlı Girdi/Çıktı kesintilerini yapılandır */
  M_PSP_WRITE_REGISTER_32(RGPIO_INTE, 0x10000);       /* RGPIO_INTE */
  M_PSP_WRITE_REGISTER_32(RGPIO_PTRIG, 0x10000);      /* RGPIO_PTRIG */
  M_PSP_WRITE_REGISTER_32(RGPIO_INTS, 0x0);           /* RGPIO_INTS */
  M_PSP_WRITE_REGISTER_32(RGPIO_CTRL, 0x1);           /* RGPIO_CTRL */
}


int main(void)
{
  int count=0, i;

  /* KESİCİ SİSTEMİNE İLK DEĞERİNİ VER */
  DefaultInitialization();                            /* Varsayılan ilk değerlendirme */
  pspExtInterruptsSetThreshold(5);                    /* Kesintilerin eşiğini 5'e ayarla */

  /* KESİCİ DOĞRUSU IRQ4'E İLK DEĞERİNİ VER */
  ExternalIntLine_Initialization(4, 6, GPIO_ISR);     /* IRQ4 doğrusuna 6 önceliğiyle ilk değerini ver. GPIO_ISR'ı Kesinti Hizmet Yordamı olarak ayarla */
  M_PSP_WRITE_REGISTER_32(Select_INT, 0x1);           /* Genel-Amaçlı Girdi/Çıktı kesintisini IRQ4 kesinti doğrusuna bağla */

  /* ÇEVRE BİRİMLERİNE İLK DEĞERLERİNİ VER */
  GPIO_Initialization();                              /* Genel-Amaçlı Girdi/Çıktıya ilk değerini ver */
  M_PSP_WRITE_REGISTER_32(SegEn_ADDR, 0x0);           /* 7-Kesimli Ekranlara ilk değerlerini ver */

  /* KESİCİLERİ ETKİNLEŞTİR */
  pspInterruptsEnable();                              /* mstatus CSR'da bütün kesintileri etkinleştir */
  M_PSP_SET_CSR(D_PSP_MIE_NUM, D_PSP_MIE_MEIE_MASK);  /* mie CSR'da dış kesintileri etkinleştir */

  while (1) {
    /* 7-Kesimli Ekranları arttır */
    M_PSP_WRITE_REGISTER_32(SegDig_ADDR, count);
    count++;

    /* Geciktir */
    for(i=0;i<50000000;i++);
  }
}

